    TITLE       "Micro-OPAZ"
    SUBTITLE    "v1.08"
    RADIX       HEX
    LIST        p=16c77
    INCLUDE     p16c77.inc
;
; == Notes ==
;
; - Quelques optimisations en vitesse excution (et pas taille de code en mmoire)
;    (Exeption faite des initialisations)


; Note quelques annes plus tard... Les commentaires ne sont vraiment par pertinents, et bourrs de fautes...



;
; Dclaration des variables utiles
;

Compteur16	equ	0x20		; Compteur pour le multiplexeur 4067 16 ports.
Compteur8	equ	0x21		; Compteur pour le multiplexeur 4051 8 ports.
TableAN16	equ	0x22		; Table des valeurs du multiplexeur 4067 16 ports.
TableAN8	equ	0x32		; Table des valeurs du multiplexeur 4051 8 ports.
resAN		equ	0x3A		; Resultat de la convertion AN.
resRx		equ	0x3B		; Rsultat de la rception USART
MidiData	equ	0x3C		; Table des valeurs MIDI
MidiCompteur	equ	0x3F	; Compteur du buffer MIDI
Save_W		equ	0x40	; Sauvegarde du registre W
Save_Status	equ	0x41		; Sauvegarde du registre Status
Save_PcLath	equ	0x42		; Sauvegarde du registre PcLath
Save_FSR	equ	0x43		; Sauvegarde du registre FSR
vDelay		equ	0x44	; Variable pour le dlais d'attente conversion AN

	org	0x000			; Reset Vector.
	goto	Start

	org	0x004			; Interruption Vector.
	goto	Interrupt




	; ====================
	; == Initialisation ==
	; ====================
	
Start					; Entre du programme.
	bcf	Status, RP1		; RP1 reste inchang tout au long du programme.
	bcf	Status, RP0		; Selectionne le Bank 0.
	clrf	PortB			; Efface le Port B.
	clrf	PortC			; Efface le Port C.
	clrf	PortD			; Efface le Port D.
	
	bsf	Status, RP0		; Slectionne le Bank 1.
	clrf	TrisB			; Initialise la Direction du Port B (out).
	clrf	TrisC			; Initialise la Direction du Port C (out).
	clrf	TrisD			; Initialise la Direction du Port D (out).

	call	InitAN			; Initialisation pour l'acquisition.
	call	InitRxTx		; Initialisation pour la trasmission.
	call	InitLeds		; Initialisation pour les leds

	bcf	Status,	RP0	; Selectionne le Bank 0.
	clrf 	Compteur16		; Met le compteur16 a 0.
	clrf 	Compteur8		; Met le compteur8 a 0.
	clrf	MidiCompteur	; Met le compteur Midi  0.

; !!! DEBUG !!!
	movlw	0x00
	;movlw	0xC0		; Slectionne les interuption GIE et PEIE.
	movwf	IntCon		;  1100 000
	bcf	Status,	IRP	; Bank 0-1 slectionn pour l'adressage indirect.

	movlw	0x02			; Utilis uniquement pour l'instruction addwf PCL, F
	movwf	PCLATH		;  qui ne sont pas dans le bon segement de mmoire




	; =======================
	; == Boucle Principale ==
	; =======================

MainLoop
LoopCompteur16
	movf	Compteur16, W	; Met le nombre du compteur16 dans 
	movwf	PortB			;  le Port B pour les multiplexeurs.
	call	AN16Start		; Appel de la routine de traitement.
	bcf	Status,	RP0	; Selectionne le Bank 0. (banks changs par les fonctions)
	incf	Compteur16, F	; Increment de Compteur.
	movf	Compteur16, W	; 
	sublw	0x0F			; ( 15 - Compteur ) -> W
	btfss	Status,	Z	; Si le resultat vaut zero, alors on saute le goto et on reinitialise le Compteur.
	goto	LoopCompteur16	; Sinon, on boucle.

LoopCompteur8
	movlw	b'11111000'		; Remplacement d'un movf  Compteur8, W & movwf PortC
	andwf	PortC			;   cause d'un conflit PortC et RXTX
	movf	Compteur8, W	;  Efface les bit 0-2 du PortC
	iorwf	PortC			;  Copie les bit 0-3 du compteur sue le PortC

	call	AN8Start		; Appel de la routine de traitement.
	bcf	Status,	RP0	; Selectionne le Bank 0. (banks changs par les fonctions)
	incf	Compteur8, F	; Increment de Compteur.
	movf	Compteur8, W	; 
	sublw	0x05			; ( 5 - Compteur ) -> W  Car 5? bouttons !!! DEBUG !!!
	btfss	Status, Z		; Si le resultat vaut zero, alors on saute le goto et on reinitialise le Compteur.
	goto	LoopCompteur8	; Sinon, on boucle.

	clrf	Compteur16
	clrf	Compteur8

	goto	MainLoop




	; ===============
	; == Interrupt ==
	; ===============

Interrupt
	movwf	Save_W		; PUSH de [W, Status, PcLath]
	movf	Status,	W 	; Sauvegarde du registre car l'interruption peu survenir n'importe quant.
	movwf	Save_Status		;
	clrf	Status 		;
	movf	PcLath,	W 	;
	movwf	Save_PcLath 	;
	clrf	PcLath		;
	movf	FSR,	W 		;
	movwf	Save_FSR 		;


	bcf	Status,	RP0	; Slectionne le Bank 0.
	btfss	PIR1,	RCIF		; Teste si c'est l'interruption de rception.
	goto	InterruptEnd	; Si ce n'est pas le cas, on quitte.

	btfsc	RCSTA,	FERR	; On teste le bit de Framing Error.
	goto	errorRecption	; 
	btfsc	RCSTA,	OERR	; On teste le bit d'Overrun Error.
	goto	errorRecption	;

	movf	RCREG,	W	; On sauvegarde le registre de l'USART dans
	movwf	ResRx			;  ResRx.
	call	processRX		; Appelle la routine qui traite txRes.

InterruptEnd
	movf	Save_FSR, W		; POP des registres sauvegard
	movwf	FSR 			;
	movf	Save_PcLath, W	;
	movwf	PcLath 		;
	movf	Save_Status, W	;
	movwf	Status		;
	movf	Save_W,	W	;

	retfie

ErrorRecption
	bcf	Status,	RP0	; Slectionne le Bank 0.
	bcf	RCSTA,	SPEN	; Ce bit est mis losqu'il y a une erreur,
					;  il faut le remettre a 0 pour la prochaine rception.		
	goto	InterruptEnd




	; ===========================
	; == Routines du Programme ==
	; ===========================


	; ------------
	; -- InitAN --
	; ------------

InitAN				; Initalisation du module de convertion AN.
	bsf	Status,	RP0	; Slectionne le Bank 1.
	clrf	ADCON1		; Toutes les entres en mode analogique.

	bcf	Status,	RP0	; Slectionne le Bank 0.
					; Cette partie du code n'a pas t optimis car elle n'est excut qu'une seule fois
					; (Initialisation), et pour des raisons de comprhention.
	bcf	ADCON0,	ADCS0	; Slection de l'horloge de conversion.
	bsf	ADCON0,	ADCS1	; -> Fosc / 32
					; ADCS0 = 0 (bit 6)
					; ADCS1 = 1 (bit 7)

	bcf	ADCON0, CHS0	; Channel 0: CHS0 = 0 (bit 3).
	bcf	ADCON0, CHS1	;            CHS1 = 0 (bit 4).
	bcf	ADCON0, CHS2	;            CHS2 = 0 (bit 5).

	bsf	ADCON0, ADON	; Active le module de conversion (bit 0).

	return		




	; --------------
	; -- InitRxTx --
	; --------------

InitRxTx
	; Rx/Tx en mode Asynchrone
	bsf	Status,	RP0	; Slectionne le Bank 1
	movlw	0x27			; Coefficient pour le 'baud rate generator'
					;  Baud Rate = Fosc / (16*(X+1)) 
					; Pour la norme MIDI avec un Oscillateur a 20.000 MHz
					;  31250 = 20000000 / (16*(39+1))
					;  Coef de 39, soit 0x27
					;
	movwf	SPBRG			; Enregistre la configuration.
	bsf	TXSTA,	BRGH	; Mode High Speed.
	bcf	TXSTA,	SYNC	; Mode Asynchrone.
	bcf	TXSTA,	TX9	; On dsactive la prise en compte du 9eme bit en mission.
	bsf	PIE1,	RCIE		; Active l'interruption du port srie USART en rception.


	bcf	Status,	RP0	; Slectionne le Bank 0.
	bsf	RCSTA,	SPEN	; Active le port serie USART (RX/TX). (Serial Port Enable)
	bcf	RCSTA,	RX9	; On dsactive la prise en compte du 9eme bit en rception.
	bsf	RCSTA,	CREN	; Active la rception

	return




	; --------------
	; -- InitLEDS --
	; --------------

InitLEDS
	bcf	Status,	RP0	; Selectionne le Bank 0.
	movlw	0x00			; Led 0 - Off
	movwf	PortD			;
	movlw	0x01			; Led 1 - Off
	movwf	PortD			;
	movlw	0x02			; Led 2 - Off
	movwf	PortD			;
	movlw	0x03			; Led 3 - Off
	movwf	PortD			;
	movlw	0x04			; Led 4 - Off
	movwf	PortD			;
	movlw	0x05			; Led 5 - Off
	movwf	PortD			;
	movlw	0x06			; Led 6 - Off
	movwf	PortD			;
	movlw	0x07			; Led 7 - Off
	movwf	PortD			;
	movlw	0x10			; Passe en mode Mmory (Qn-1)
	movwf	PortD			;

	return




	; --------------
	; -- MakeAN16 --
	; --------------

AN16Start
	bcf	Status,	RP0	; Selectionne le Bank 0.
	bcf	ADCON0,	CHS0	; Channel 0.
	clrf	ADRES			; Efface l'ancienne valeur d'acquisition.
	call	delayAN		; Laisse en temps ncessaire pour le sampling.
	bsf	ADCON0,	GO	; Commence la convertion en placant le bit
					;  GO de ADCON0 a 1.
AN16Wait
	btfsc	ADCON0, GO		; La conversion AN est finie si le bit GO de ADCON0 est mise a 0 par le processeur.
	goto	AN16Wait		; Le bit est a 1 alors on boucle.

	; La conversion est ici termine.
	movf	ADRES,	W	; Sauvegarde du rsultat 
	movwf	resAN			;  dans ResAN
	movlw	TableAN16		; pointeurFSR = adresseTable + Compteur.
	addwf	Compteur16, W	;  
	movwf	FSR			;  Adresse Table(i) dans FSR.

	movf	resAN,	W	; 
	subwf	INDF,	W		; ( valeurTable - valeurActuelle ) -> W 
					; INDF = valeur du pointeur FSR.	

	; On teste tout d'abord l'glit.
	btfsc	Status, Z		; Cas d'galit, valeur inchangee.
	goto	AN16NoTransmit	;  On quitte vers noTransmit.

	; Traitement des cas de Non-Egalite:
	; W contient a prsent la valeur de la soustraction
	btfss	Status,	C	; On teste si la valeur est negative.
	goto	AN16Negative	;  On continue  negativeComp si oui.

AN16Positive				; Traitement des cas Positifs:
					; Pour viter la transmission intensive lie a l'ocsillation permanente du dernier
					;  bit, on effectue un soustration de 2, qui qui garantie le changement rel de valeur.
					;
					; On restrain alors la precsion a 7 bits, au lieu des 8 bits initiaux.
					; Sans importance car le midi traite du 7 bits. (valeur de 0-127)
	sublw	0x01			; ( 1 - W ) -> W'  // 00 + 1 = 0x01
	btfss	Status, C		;
	goto	AN16Transmit	; Si le resultat est gal a zro ou superieur, alors on transmet la nouvelle valeur.
	goto	AN16NoTransmit	; Dans la cas contraire, (valeur negative) on ne transmet rien.

AN16Negative
	; Ici, on a un W 'ngatif' donc du genre 0xFF.
	sublw	0xFE			; ( 254 - W ) -> W' // FF - 1 = 0xFE
	btfss	Status, C		;
	goto	AN16NoTransmit	; 

AN16Transmit
	movf	resAN,	W	; Sauvegarde du rsultat dans 
	movwf	INDF			;  la Table(Compteur).
	sublw	0xFF			; Putain de connerie de potar invers !
	movwf	resAN			;  Conversion de valeur: FF -> 00
	bcf	Status,	C	; Decalage d'un bit vers la droite
	rrf	resAN,	F	; ce qui ramne la precision a 7 bits, soit 127.
	call    Tx16Start		; Transmission via RS232.

AN16NoTransmit
	return




	; -------------
	; -- MakeAN8 --
	; -------------
AN8Start
	bcf	Status,	RP0	; Selectionne le Bank 0.
	bsf	ADCON0,	CHS0	; Channel 1.
	clrf	ADRES			; Efface l'ancienne valeur d'acquisition.
	call	delayAN		; Laisse en temps ncessaire pour le sampling.
	bsf	ADCON0, GO		; Commence la convertion en placant le bit
					;  GO de ADCON0 a 1.
AN8Wait
	btfsc	ADCON0, GO		; La conversion AN est finie si le bit GO de ADCON0 est mise a 0 par le processeur.
	goto	AN8Wait		; Le bit est a 1 alors on boucle.

	; L'aquisition AN est ici termine.
	movf	ADRES,	W	; Sauvegarde du rsultat 
        sublw	0x80		; (128 - W) -> W
	btfss	Status, C		; 128 - 255(on)  -> -127 : valeur ngative = ON
        goto    AN8On		; 128 - 000(off) ->  128 : valeur positive = OFF

AN8Off
	movlw	0x00			; Valeur OFF pour le controleur
	movwf	resAN			; 

	movlw	TableAN8		; pointeurFSR = adresseTableAN8 + valeurCompteur.
	addwf	Compteur8, W	;  
	movwf	FSR			;  Adresse TableAN8(Compteur8) dans FSR.

	btfss	INDF, 0
	goto	AN8NoTransmit

	movf	resAN,	W	; Sauvegarde du rsultat dans 
	movwf	INDF			;  la TableAN8(Compteur8).
	call    Tx8Start		; Transmission via Midi.
	return

AN8On
	movlw	0x01			; Valeur ON pour le controleur
	movwf	resAN			;  

	movlw	TableAN8		; pointeurFSR = adresseTableAN8 + valeurCompteur.
	addwf	Compteur8, W	;  
	movwf	FSR			;  Adresse TableAN8(Compteur8) dans FSR.

	btfsc	INDF, 0
	goto	AN8NoTransmit

	movf	resAN,	W	; Sauvegarde du rsultat dans 
	movwf	INDF			;  la TableAN8(Compteur8).
	call    Tx8Start		; Transmission via Midi.
	return

AN8NoTransmit
	return




	; ----------------
	; -- TxRes_AN16 --
	; ----------------

Tx16Start
	bsf 	Status, RP0		; Selectionne le Bank 1.
	bsf	TXSTA, 	TXEN	; Active la Transmission.

	; Transmission du header: Control Change
	bcf	Status, RP0		; Selectionne le Bank 0.
	movlw	0xB0			; On transmet la valeur 1011 0000 : Changement de controleur sur Canal Midi 1.
	movwf	TXREG			; Place la valeur dans le registre de transmission, et commence la convertion.
	bsf	Status, RP0		; Selectionne le Bank 1.
Tx16Wait0
	btfss	TXSTA,	TRMT	; Boucle d'attente du vidage du registre TSR.
	goto	Tx16Wait0		; Sort de la boucle quand TSR est vide (TSR = 1).

	; Transmission du numero du controleur.
	bcf	Status, RP0		; Selectionne le Bank 0.
	movf	Compteur16, W	; On transmet la valeur du compteur.
        call    TranslateNum16; Appel de la fct de rordonnancement.
	movwf	TXREG			; Place la valeur dans le registre de transmission, et commence la convertion.
	bsf	Status, RP0		; Selectionne le Bank 1.
Tx16Wait1
	btfss	TXSTA,	TRMT	; Boucle d'attente du vidage du registre TSR.
	goto	Tx16Wait1		; Sort de la boucle quand TSR est vide (TSR = 1).


	; Transmission de la valeur du controleur.
	bcf	Status, RP0		; Selectionne le Bank 0.
	movf	resAN,	W	; On transmet la valeur de la conversion AN.
	movwf	TXREG			; Place la valeur dans le registre de transmission, et commence la convertion.
	bsf	Status, RP0		; Selectionne le Bank 1.
Tx16Wait2
	btfss	TXSTA,	TRMT	; Boucle d'attente du vidage du registre TSR.
	goto	Tx16Wait2		; Sort de la boucle quand TSR est vide (TSR = 1).

	bsf	TXSTA, 	TXEN	; Dsactive la Transmission : Fin de Transmission
	return




	; ---------------
	; -- TxRes_AN8 --
	; ---------------

Tx8Start
	bsf 	Status, RP0		; Selectionne le Bank 1.
	bsf	TXSTA, 	TXEN	; Active la Transmission.

	; Transmission du Header Note et canal Midi
	bcf	Status, RP0		; Selectionne le Bank 0.
	movlw	0x90			;
	movwf	TXREG			; Place la valeur dans le registre de transmission, et commence la convertion.
	bsf	Status, RP0		; Selectionne le Bank 1.
Tx8Wait0
	btfss	TXSTA,	TRMT	; Boucle d'attente du vidage du registre TSR.
	goto	Tx8Wait0		; Sort de la boucle quand TSR est vide (TSR = 1).


	; Transmission de la hauteur note (en fonction du numro de touche)
	bcf	Status, RP0		; Selectionne le Bank 0.
	movf	Compteur8, W	; On transmet la valeur du compteur.
	call	TranslateNum8	; Appel de la fct de rordonnancement.
	movwf	TXREG			; Place la valeur dans le registre de transmission, et commence la conversion.
	bsf	Status, RP0		; Selectionne le Bank 1.
Tx8Wait1
	btfss	TXSTA,	TRMT	; Boucle d'attente du vidage du registre TSR.
	goto	Tx8Wait1		; Sort de la boucle quand TSR est vide (TSR = 1).


	; Transmission de la vlocit Note On/Off
	bcf	Status, RP0		; Selectionne le Bank 0.
	btfsc	resAN, 0		;
	goto	Tx8NoteOn

Tx8NoteOff
	movlw	0x00			; noteOff
	goto	Tx8Suite
Tx8NoteOn
	movlw	0x5A			; noteOn

Tx8Suite
	movwf	TXREG			; Place la valeur dans le registre de transmission, et commence la convertion.
	bsf	Status, RP0		; Selectionne le Bank 1.
Tx8Wait2
	btfss	TXSTA,	TRMT	; Boucle d'attente du vidage du registre TSR.
	goto	Tx8Wait2		; Sort de la boucle quand TSR est vide (TSR = 1).

	bsf	TXSTA, 	TXEN	; Dsactive la Transmission : Fin de Transmission
	return




	; -------------
	; -- delayAN --
	; -------------

delayAN
; !!! DEBUG !!! Facultatif si l'on considre le code en dessous de l'appel... A vrif

					; On doit deja etre dans le Bank 0 pour la variable vDelay.
	movlw	0x20			; 10S de delay calcul pour un Pic a 20MHz.
	movwf	vDelay		;
delayLoop
	decfsz	vDelay,F    ; Test et dcrementation.
	goto	delayLoop		; Boucle tant que l'on est pas a zero.
	return




	; ---------------
	; -- processRX --
	; ---------------

ProcessRX
      bcf     Status, RP0     ; Selectionne le Bank 0.
      movlw   MidiData        ; pointeurFSR = adresseMidiData + MidiCompteur.
	addwf	MidiCompteur, W	;  
	movwf	FSR			;  Adresse MidiData(MidiCompteur) dans FSR.
      movf    resRx,  W       ; Sauvegarde resTx dans MidiData(MidiCompteur)
      movwf   INDF            ;  INDF = [MidiData(MidiCompteur)]
      incf    MidiCompteur, F ; Incrmente le compteur midi

	movf	MidiCompteur, W
	sublw	0x03			; 3 - W -> W
	btfss	Status,	Z	; S
	return

	; On a rempli la structure Midi (3 octets) : R-mission Midi + Analyse pour LEDs
	clrf	MidiCompteur	; On rinitialise le compteur midi  0
	call	MidiReEmit
	call	MidiCheckLeds
	return




	; ----------------
	; -- MidiReEmit --
	; ----------------

MidiReEmit
	bsf 	Status, RP0		; Selectionne le Bank 1.
	bsf	TXSTA, 	TXEN	; Active la Transmission.

	; Transmission 
	bcf	Status, RP0		; Selectionne le Bank 0.
	movf	(MidiData+0x00), W; 
	movwf	TXREG			; Place la valeur dans le registre de transmission, et commence la convertion.
	bsf	Status, RP0		; Selectionne le Bank 1.
MidiReEmit_Wait0
	btfss	TXSTA,	TRMT	; Boucle d'attente du vidage du registre TSR.
	goto	MidiReEmit_Wait0	; Sort de la boucle quand TSR est vide (TSR = 1).

	; Transmission 
	bcf	Status, RP0		; Selectionne le Bank 0.
	movf	(MidiData+0x01), W; 
	movwf	TXREG			; Place la valeur dans le registre de transmission, et commence la convertion.
	bsf	Status, RP0		; Selectionne le Bank 1.
MidiReEmit_Wait1
	btfss	TXSTA,	TRMT	; Boucle d'attente du vidage du registre TSR.
	goto	MidiReEmit_Wait1	; Sort de la boucle quand TSR est vide (TSR = 1).


	; Transmission 
	bcf	Status, RP0		; Selectionne le Bank 0.
	movf	(MidiData+0x02), W; 
	movwf	TXREG			; Place la valeur dans le registre de transmission, et commence la convertion.
	bsf	Status, RP0		; Selectionne le Bank 1.
MidiReEmit_Wait2
	btfss	TXSTA,	TRMT	; Boucle d'attente du vidage du registre TSR.
	goto	MidiReEmit_Wait2	; Sort de la boucle quand TSR est vide (TSR = 1).

	bsf	TXSTA, 	TXEN	; Dsactive la Transmission : Fin de Transmission.
	return




	; -------------------
	; -- MidiCheckLeds --
	; -------------------

MidiCheckLeds
	movf	(MidiData+0x00), W; 1er octet de MidiData.
	sublw	b'10000000'		; Note OFF pour le canal Midi 1.
	btfsc	Status,	Z	; Teste l'galit de la soustreaction.
	goto	MidiLedOff		;  si la soustraction vaut 0 (donc galit des deux nombres) on saute.
	movf	(MidiData+0), W	; 1er octet de MidiData.
	sublw	b'10010000'		; Note OFF pour le canal Midi 1.
	btfsc	Status,	Z	; Teste l'galit de la soustreaction.
	goto	MidiLedOn		;  si la soustraction vaut 0 (donc galit des deux nombres) on saute.
	return

MidiLedOff
	movf	(MidiData+0x01), W; 2me octet de MidiData.
	sublw	0x04			; Note de base (4 - W) -> W [5 leds utiliss].
	btfss	Status,	C	; Teste si le rsultat est ngatif.
	goto	noteInvalide	;  Si la soutraction est ngative, on dpasse les 5 leds, donc on ne traite pas.
	; Note Valide
	movf	(MidiData+0x01), W; 2me octet de MidiData.
	iorlw	b'00001000'		; OU Logique avec le masque pour allumer la LED.
	movwf	PortD			; Ecriture des valeurs sur le port.

MidiLedOn
	movf	(MidiData+0x01), W; 2me octet de MidiData.
	sublw	0x04			; Note de base (4 - W) -> W [5 leds utiliss].
	btfss	Status,	C	; Teste si le rsultat est ngatif.
	goto	noteInvalide	;  Si la soutraction est ngative, on dpasse les 5 leds, donc on ne traite pas.
	; Note Valide
	movf	(MidiData+0x01), W; 2me octet de MidiData.
	movwf	PortD			; Ecriture des valeurs sur le port pour teindre les leds.

noteInvalide
	return




	; -------------
	; -- SetLED1 --
	; -------------

SetLED1
	bcf	Status,	RP0	; Selectionne le Bank 0.
	movlw	0x0C			; Led 0x00
	movwf	PortD			;
	movlw	0x10			; Passe en mode Mmory (Qn-1)
	movwf	PortD			;
	return




	; -------------
	; -- SetLED2 --
	; -------------

SetLED2
	bcf	Status,	RP0	; Selectionne le Bank 0.
	movlw	0x0B			; Led 0x01 
	movwf	PortD			;
	movlw	0x10			; Passe en mode Mmory (Qn-1)
	movwf	PortD			;
	return




	; -------------
	; -- SetLED3 --
	; -------------

SetLED3
	bcf	Status,	RP0	; Selectionne le Bank 0.
	movlw	0x08			; Led 0x02
	movwf	PortD			;
	movlw	0x10			; Passe en mode Mmory (Qn-1)
	movwf	PortD			;
	return




	; -------------
	; -- SetLED4 --
	; -------------

SetLED4
	bcf	Status,	RP0	; Selectionne le Bank 0.
	movlw	0x09			; Led 0x03
	movwf	PortD			;
	movlw	0x10			; Passe en mode Mmory (Qn-1)
	movwf	PortD			;
	return




	; -------------
	; -- SetLED5 --
	; -------------

SetLED5
	bcf	Status,	RP0	; Selectionne le Bank 0.
	movlw	0x0A			; Led 0x04
	movwf	PortD			;
	movlw	0x10			; Passe en mode Mmory (Qn-1)
	movwf	PortD			;
	return




	; ---------------
	; -- ReSetLED1 --
	; ---------------

ReSetLED1
	bcf	Status,	RP0	; Selectionne le Bank 0.
	movlw	0x04			; Led 0x00
	movwf	PortD			;
	movlw	0x10			; Passe en mode Mmory (Qn-1)
	movwf	PortD			;
	return




	; ---------------
	; -- ReSetLED2 --
	; ---------------

ReSetLED2
	bcf	Status,	RP0	; Selectionne le Bank 0.
	movlw	0x03			; Led 0x01 
	movwf	PortD			;
	movlw	0x10			; Passe en mode Mmory (Qn-1)
	movwf	PortD			;
	return




	; ---------------
	; -- ReSetLED3 --
	; ---------------

ReSetLED3
	bcf	Status,	RP0	; Selectionne le Bank 0.
	movlw	0x00			; Led 0x02
	movwf	PortD			;
	movlw	0x10			; Passe en mode Mmory (Qn-1)
	movwf	PortD			;
	return




	; ---------------
	; -- ReSetLED4 --
	; ---------------

ReSetLED4
	bcf	Status,	RP0	; Selectionne le Bank 0.
	movlw	0x01			; Led 0x03
	movwf	PortD			;
	movlw	0x10			; Passe en mode Mmory (Qn-1)
	movwf	PortD			;
	return




	; ---------------
	; -- ReSetLED5 --
	; ---------------

ReSetLED5
	bcf	Status,	RP0	; Selectionne le Bank 0.
	movlw	0x02			; Led 0x04
	movwf	PortD			;
	movlw	0x10			; Passe en mode Mmory (Qn-1)
	movwf	PortD			;
	return




	; *******************
	; ** Mmoire 0x200 **
	; *******************
	org	0x200


	; -------------------
	; -- TranslateNum8 --
	; -------------------

TranslateNum8
	; Fonction de Convertion d'un numro en un autre.
	; Prend W en entre, et sort le numro correspondant dans W.
	addwf 	PCL,	F		;           Avance le PC de W
        retlw	0x35			; (3) - F   Valeur de rordonnancement
        retlw	0x34			; (2) - E
        retlw	0x32			; (1) - D
        retlw	0x37			; (4) - G
        retlw	0x30			; (0) - C
        retlw	0x0F			; ()
        retlw	0x0F			; ()
        retlw	0x0F			; ()




	; --------------------
	; -- TranslateNum16 --
	; --------------------

TranslateNum16
	; Fonction de Convertion d'un numro en un autre.
	; Prend W en entre, et sort le numro correspondant dans W.
	addwf 	PCL,	F		; Avance le PC de W
        retlw	0x04			; Valeur de rordonnancement
        retlw	0x03			;
        retlw	0x01			;
        retlw	0x08			;
        retlw	0x06			;
        retlw	0x0D			;
        retlw	0x0B			;
        retlw	0x05			;
        retlw	0x0A			;
        retlw	0x00			;
        retlw	0x0C			;
        retlw	0x0E			;
        retlw	0x07			;
        retlw	0x09			;
        retlw	0x02			;
        retlw	0x0F			;




	END
